@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 # -*- coding: utf-8 -*-
 #
 # «recovery_basic_gtk» - Dell Recovery Media Generator
@@ -27,10 +27,10 @@ import os
 import subprocess
 import dbus
 
-import gtk
+from gi.repository import Gtk
 
 from Dell.recovery_gtk import DellRecoveryToolGTK, translate_widgets
-from Dell.recovery_common import (find_partitions, find_burners, UIDIR,
+from Dell.recovery_common import (find_partitions, find_burners, UIDIR, SVGDIR,
                                   increment_bto_version,
                                   dbus_sync_call_signal_wrapper,
                                   fetch_output)
@@ -46,18 +46,28 @@ class BasicGeneratorGTK(DellRecoveryToolGTK):
 
         #Run the normal init first
         #This sets up lots of common variables as well as translation domain
-        DellRecoveryToolGTK.__init__(self, recovery)
+        DellRecoveryToolGTK.__init__(self, recovery, utility)
 
         #init the UI and translate widgets/connect signals
-        self.widgets = gtk.Builder()
+        self.widgets = Gtk.Builder()
         self.widgets.add_from_file(os.path.join(UIDIR,
                                    'recovery_media_creator.ui'))
-        gtk.window_set_default_icon_from_file('/usr/share/pixmaps/dell-dvd.svg')
+
+        #insert pages in reverse order
+        wizard = self.widgets.get_object('wizard')
+        titles = {'conf_page': _("Confirm Selections"),
+                 'media_type_page': _("Media Type")
+                }
+        for page in ['conf_page', 'media_type_page']:
+            wizard.insert_page(self.widgets.get_object(page), 0)
+            wizard.set_page_title(wizard.get_nth_page(0), titles[page])
+
+        Gtk.Window.set_default_icon_from_file(os.path.join(SVGDIR, 'dell-dvd.svg'))
         translate_widgets(self.widgets)
         self.widgets.connect_signals(self)
 
         #setup spinner
-        self.spinner = gtk.Spinner()
+        self.spinner = Gtk.Spinner()
         self.widgets.get_object('spinner_box').add(self.spinner)
         self.widgets.get_object('spinner_box').show_all()
 
@@ -69,7 +79,7 @@ class BasicGeneratorGTK(DellRecoveryToolGTK):
 
         try:
             self.release = fetch_output(['lsb_release', '-r', '-s']).strip('\n')
-            self.distributor = fetch_output(['lsb_release', '-i', '-s']).strip('\n')
+            self.distributor = fetch_output(['lsb_release', '-i', '-s']).strip('\n').lower()
         except RuntimeError:
             self.release = '0.00'
             self.distributor = 'unknown'
@@ -99,7 +109,7 @@ class BasicGeneratorGTK(DellRecoveryToolGTK):
         if self.up and self.rp:
             return True
 
-        (self.up, self.rp) = find_partitions(self.up, self.rp)
+        (self.up, self.rp) = find_partitions()
 
         return self.rp
 
@@ -114,10 +124,10 @@ class BasicGeneratorGTK(DellRecoveryToolGTK):
                                             'query_iso_information',
                                             {'report_iso_info': self.update_version_gui},
                                             self.rp)
-            except dbus.DBusException, msg:
+            except dbus.DBusException as msg:
                 self.toggle_spinner_popup(False)
-                parent = self.widgets.get_object('wizard')
-                self.dbus_exception_handler(msg, parent)
+                transient_for = self.widgets.get_object('wizard')
+                self.dbus_exception_handler(msg, transient_for)
                 return
             finally:
                 self.toggle_spinner_popup(False)
@@ -157,8 +167,7 @@ class BasicGeneratorGTK(DellRecoveryToolGTK):
                 self.widgets.get_object('action').set_text( \
                                                        _("Building Base image"))
                 function = 'create_' + self.distributor
-                args = (False,
-                        self.up,
+                args = (self.up,
                         self.rp)
 
             #all functions require this at the end
@@ -170,10 +179,10 @@ class BasicGeneratorGTK(DellRecoveryToolGTK):
                                 {'report_progress':self.update_progress_gui},
                                 *args)
                 self.update_progress_gui(_("Opening Burner"), 1.00)
-            except dbus.DBusException, msg:
-                parent = self.widgets.get_object('progress_dialog')
+            except dbus.DBusException as msg:
+                transient_for = self.widgets.get_object('progress_dialog')
                 fallback = self.widgets.get_object('wizard')
-                self.dbus_exception_handler(msg, parent, fallback)
+                self.dbus_exception_handler(msg, transient_for, fallback)
                 return
 
         self.burn()
@@ -198,8 +207,8 @@ class BasicGeneratorGTK(DellRecoveryToolGTK):
         body = _("If you would like to archive another copy, the generated \
 image has been stored under the filename:\n") + \
 os.path.join(self.path, self.image)
-        self.show_alert(gtk.MESSAGE_INFO, header, body,
-            parent=self.widgets.get_object('progress_dialog'))
+        self.show_alert(Gtk.MessageType.INFO, header, body,
+            transient_for=self.widgets.get_object('progress_dialog'))
 
         self.destroy(None)
 
@@ -212,8 +221,8 @@ os.path.join(self.path, self.image)
             header = _("Unable to proceed")
             inst = _("System does not appear to contain Dell factory installed \
 partition layout.")
-            self.show_alert(gtk.MESSAGE_ERROR, header, inst,
-                parent=self.widgets.get_object('wizard'))
+            self.show_alert(Gtk.MessageType.ERROR, header, inst,
+                transient_for=self.widgets.get_object('wizard'))
             return
 
         if DellRecoveryToolGTK.top_button_clicked(self, widget):
@@ -225,7 +234,7 @@ partition layout.")
     def check_close(self, widget, args=None):
         """Asks the user before closing the dialog"""
         response = self.widgets.get_object('close_dialog').run()
-        if response == gtk.RESPONSE_YES:
+        if response == Gtk.ResponseType.YES:
             self.destroy()
         else:
             self.widgets.get_object('close_dialog').hide()
@@ -234,8 +243,8 @@ partition layout.")
     def hide_progress(self):
         """Hides the progress bar"""
         self.widgets.get_object('progress_dialog').hide()
-        while gtk.events_pending():
-            gtk.main_iteration()
+        while Gtk.events_pending():
+            Gtk.main_iteration()
 
     def toggle_spinner_popup(self, force):
         wizard = self.widgets.get_object('wizard')
@@ -248,8 +257,8 @@ partition layout.")
             self.spinner.stop()
             popup.hide()
             wizard.set_sensitive(True)
-        while gtk.events_pending():
-            gtk.main_iteration()
+        while Gtk.events_pending():
+            Gtk.main_iteration()
 
     def update_version_gui(self, version, distributor, release, arch, output_text):
         """Stops any running spinners and updates GUI items"""
@@ -284,16 +293,13 @@ partition layout.")
         if progress_text != None:
             self.widgets.get_object('action').set_markup("<i>" +\
                                                         _(progress_text)+"</i>")
-        while gtk.events_pending():
-            gtk.main_iteration()
+        while Gtk.events_pending():
+            Gtk.main_iteration()
         return True
 
     def build_page(self, widget, page=None):
         """Prepares our GTK assistant"""
-
         if page == self.widgets.get_object('media_type_page'):
-            self.widgets.get_object('wizard').set_page_title(page,
-                                                         _("Choose Media Type"))
             #fill in command line args
             if self.media == "dvd":
                 self.widgets.get_object('dvdbutton').set_active(True)
@@ -328,7 +334,7 @@ partition layout.")
             text  = ''
             text += "<b>" + _("Media Type: ") + '</b>' + burn_type + '\n'
             if self.rp:
-                if not "/dev" in self.rp:
+                if not self.rp.startswith(b'/dev'):
                     text += "<b>" + _("Recovery Partition: ") + '</b>'
                     text += self.rp + '\n'
                 else:
@@ -339,12 +345,11 @@ partition layout.")
             self.widgets.get_object('conf_text').set_markup(text)
 
             if page:
-                self.widgets.get_object('wizard').set_page_title(page,
-                                                        _("Confirm Selections"))
                 self.widgets.get_object('wizard').set_page_complete(page, True)
+                self.widgets.get_object('wizard').set_page_type(page, Gtk.AssistantPageType.CONFIRM)
 
 def show_question(dialog):
     """Presents the user with a question"""
     response = dialog.run()
     dialog.hide()
-    return (response == gtk.RESPONSE_YES)
+    return (response == Gtk.ResponseType.YES)
