@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 # -*- coding: utf-8 -*-
 #
 # «recovery_common» - Misc Functions and variables that are useful in many areas
@@ -25,7 +25,7 @@
 
 import dbus.mainloop.glib
 import subprocess
-import gobject
+from gi.repository import GLib, UDisks
 import os
 import shutil
 import re
@@ -33,6 +33,7 @@ import tempfile
 import glob
 import sys
 import datetime
+import logging
 
 ##                ##
 ##Common Variables##
@@ -41,6 +42,9 @@ import datetime
 DBUS_BUS_NAME = 'com.dell.RecoveryMedia'
 DBUS_INTERFACE_NAME = 'com.dell.RecoveryMedia'
 
+#For install time
+CDROM_MOUNT = '/cdrom'
+ISO_MOUNT = '/isodevice'
 
 #Translation Support
 DOMAIN = 'dell-recovery'
@@ -52,6 +56,12 @@ if os.path.isdir('gtk') and 'DEBUG' in os.environ:
 else:
     UIDIR = '/usr/share/dell'
 
+#SVG file directory
+if os.path.isdir('gtk') and 'DEBUG' in os.environ:
+    SVGDIR = 'gtk'
+else:
+    SVGDIR = '/usr/share/pixmaps'
+
 
 #Supported burners and their arguments
 DVD_BURNERS = { 'brasero':['-i'],
@@ -60,15 +70,6 @@ USB_BURNERS = { 'usb-creator':['-n', '--iso'],
                 'usb-creator-gtk':['-n', '--iso'],
                 'usb-creator-kde':['-n', '--iso'] }
 
-if 'INTRANET' in os.environ:
-    URL = "humbolt.us.dell.com/pub/linux.dell.com/srv/www/vhosts/linux.dell.com/html"
-else:
-    URL = "linux.dell.com"
-
-GIT_TREES = { 'ubuntu': 'git://' + URL + '/ubuntu-fid.git',
-              'redhat': 'http://humbolt.us.dell.com/pub/Applications/git-internal-projects/redhat-fid.git',
-            }
-
 #UP File names
 UP_FILENAMES =  [ 'upimg.bin',
                   'upimg.gz' ,
@@ -76,6 +77,9 @@ UP_FILENAMES =  [ 'upimg.bin',
                   'up.tgz'   ,
                 ]
 
+UP_LABELS = [ 'dellutility' ]
+RP_LABELS = [ 'recovery', 'install', 'os' ]
+
 ##                ##
 ##Common Functions##
 ##                ##
@@ -156,11 +160,12 @@ def check_version(package='dell-recovery'):
         cache = apt.cache.Cache()
         if cache[package].is_installed:
             return cache[package].installed.version
-    except Exception, msg:
-        print >> sys.stderr, "Error checking %s version: %s" % (package, msg)
+    except Exception as msg:
+        print("Error checking %s version: %s" % (package, msg),
+              file=sys.stderr)
         return "unknown"
 
-def process_conf_file(original, new, uuid, rp_number, ako='', recovery_text=''):
+def process_conf_file(original, new, uuid, number, ako='', recovery_text=''):
     """Replaces all instances of a partition, OS, and extra in a conf type file
        Generally used for things that need to touch grub"""
     if not os.path.isdir(os.path.split(new)[0]):
@@ -188,7 +193,7 @@ def process_conf_file(original, new, uuid, rp_number, ako='', recovery_text=''):
     #starting with 10.10, we replace the whole drive string (/dev/sdX,msdosY)
     #earlier releases are hardcoded to (hd0,Y)
     if float(release["RELEASE"]) >= 10.10:
-        rp_number = 'msdos' + rp_number
+        number = 'msdos' + number
 
     with open(original, "r") as base:
         with open(new, 'w') as output:
@@ -198,18 +203,20 @@ def process_conf_file(original, new, uuid, rp_number, ako='', recovery_text=''):
                 if "#UUID#" in line:
                     line = line.replace("#UUID#", uuid)
                 if "#PARTITION#" in line:
-                    line = line.replace("#PARTITION#", rp_number)
+                    line = line.replace("#PARTITION#", number)
                 if "#OS#" in line:
                     line = line.replace("#OS#", "%s %s" % (release["ID"], release["RELEASE"]))
                 if "#EXTRA#" in line:
                     line = line.replace("#EXTRA#", "%s" % extra_cmdline.strip())
                 output.write(line)
 
-def fetch_output(cmd, data=None):
+def fetch_output(cmd, data='', environment=os.environ):
     '''Helper function to just read the output from a command'''
     proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE,
-                                 stdin=subprocess.PIPE)
+                                 stdin=subprocess.PIPE,
+                                 env=environment,
+                                 universal_newlines=True)
     (out, err) = proc.communicate(data)
     if proc.returncode is None:
         proc.wait()
@@ -217,24 +224,9 @@ def fetch_output(cmd, data=None):
         error = "Command %s failed with stdout/stderr: %s\n%s" % (cmd, out, err)
         import syslog
         syslog.syslog(error)
-        raise RuntimeError, (error)
+        raise RuntimeError(error)
     return out
 
-def find_supported_ui():
-    """Finds potential user interfaces"""
-    desktop = { 'gnome'             : 'gnome.desktop',
-                'unity-2d'          : 'unity-2d.desktop',
-                'gnome-classic'     : 'gnome-classic.desktop',
-                'gnome-2d'          : 'gnome-2d.desktop'}
-    name =    { 'gnome'             : 'Unity (3D)',
-                'unity-2d'          : 'Unity (2D)',
-                'gnome-classic'     : 'GNOME (Classic 3D)',
-                'gnome-2d'          : 'GNOME (Classic 2D)'}
-    for item in desktop:
-        if not os.path.exists(os.path.join('/usr/share/xsessions/', desktop[item])):
-            name.pop(item)
-    return name
-
 def find_extra_kernel_options():
     """Finds any extra kernel command line options"""
     with open('/proc/cmdline', 'r') as cmdline:
@@ -244,113 +236,66 @@ def find_extra_kernel_options():
     else:
         return ''
 
-def find_factory_rp_stats():
+def find_factory_partition_stats(partition_type):
     """Uses udisks to find the RP of a system and return stats on it
-       Only use this method during bootstrap."""
-    bus = dbus.SystemBus()
+       Only use this method during bootstrap.
+       args: 'up' or 'rp'
+    """
     recovery = {}
-    udisk_bus_name = 'org.freedesktop.UDisks'
-    dev_bus_name   = 'org.freedesktop.UDisks.Device'
+    if partition_type == 'up':
+        labels = UP_LABELS
+    elif partition_type == 'rp':
+        labels = RP_LABELS
 
-    try:
-        obj = bus.get_object(udisk_bus_name, '/org/freedesktop/UDisks')
-        iface = dbus.Interface(obj, udisk_bus_name)
-        devices = iface.EnumerateDevices()
-        for check_label in ['RECOVERY', 'install', 'OS']:
-            for device in devices:
-                obj = bus.get_object(udisk_bus_name, device)
-                dev = dbus.Interface(obj, 'org.freedesktop.DBus.Properties')
-
-                if check_label == dev.Get(dev_bus_name, 'IdLabel'):
-                    recovery["label" ] = check_label
-                    recovery["device"] = dev.Get(dev_bus_name, 'DeviceFile')
-                    recovery["fs"    ] = dev.Get(dev_bus_name, 'IdType')
-                    recovery["slave" ] = dev.Get(dev_bus_name, 'PartitionSlave')
-                    recovery["number"] = dev.Get(dev_bus_name, 'PartitionNumber')
-                    recovery["parent"] = dev.Get(dev_bus_name, 'PartitionSlave')
-                    recovery["uuid"]   = dev.Get(dev_bus_name, 'IdUuid')
-                    parent_obj    = bus.get_object(udisk_bus_name, recovery["parent"])
-                    parent_dev    = dbus.Interface(parent_obj, 'org.freedesktop.DBus.Properties')
-                    recovery["size_gb"] = parent_dev.Get(dev_bus_name, 'DeviceSize') \
-                                    / 1000000000
-                    break
-            if recovery:
-                dev_obj = bus.get_object(udisk_bus_name, recovery["slave"])
-                dev = dbus.Interface(dev_obj, 'org.freedesktop.DBus.Properties')
-                recovery["slave"] = dev.Get(dev_bus_name, 'DeviceFile')
+    udisks = UDisks.Client.new_sync(None)
+    manager = udisks.get_object_manager()
+    for item in manager.get_objects():
+        block = item.get_block()
+        if not block:
+            continue
+        if block.get_cached_property('HintSystem').get_boolean():
+            check_label = block.get_cached_property("IdLabel")
+            if not check_label:
+                continue
+            if check_label.get_string().lower() in labels:
+                partition = item.get_partition()
+                recovery["label"] = check_label.get_string()
+                recovery["device"] = block.get_cached_property("Device").get_bytestring()
+                recovery["fs"] = block.get_cached_property("IdType").get_string()
+                recovery["drive"] = block.get_cached_property("Drive").get_string()
+                recovery["number"] = partition.get_cached_property("Number").unpack()
+                recovery["uuid"] = block.get_cached_property("IdUUID").get_string()
+                recovery["size_gb"] = partition.get_cached_property("Size").unpack() / 1000000000
                 break
 
-    except dbus.DBusException, msg:
-        print "%s, UDisks Failed" % str(msg)
-
+    #find parent slave node, used for dell-bootstrap
+    if "device" in recovery:
+        for item in manager.get_objects():
+            table = item.get_partition_table()
+            if not table:
+                continue
+            block = item.get_block()
+            if not block:
+                continue
+            if block.get_cached_property("Drive").get_string() == recovery["drive"]:
+                recovery["slave"] = block.get_cached_property("Device").get_bytestring().decode('utf-8')
+                break
     return recovery
 
-def find_partitions(utility, recovery):
+def find_partitions():
     """Searches the system for utility and recovery partitions"""
-    bus = dbus.SystemBus()
 
-    try:
-        #first try to use udisks, if this fails, fall back to devkit-disks.
-        obj = bus.get_object('org.freedesktop.UDisks', '/org/freedesktop/UDisks')
-        iface = dbus.Interface(obj, 'org.freedesktop.UDisks')
-        devices = iface.EnumerateDevices()
-        for device in devices:
-            dev_obj = bus.get_object('org.freedesktop.UDisks', device)
-            dev = dbus.Interface(dev_obj, 'org.freedesktop.DBus.Properties')
-
-            label = dev.Get('org.freedesktop.UDisks.Device', 'IdLabel')
-            filesystem = dev.Get('org.freedesktop.Udisks.Device', 'IdType')
-
-            if not utility and 'DellUtility' in label:
-                utility = dev.Get('org.freedesktop.UDisks.Device', 'DeviceFile')
-            elif not recovery and (('install' in label or 'OS' in label) and 'vfat' in filesystem) or \
-                            ('RECOVERY' in label and 'ntfs' in filesystem):
-                recovery = dev.Get('org.freedesktop.Udisks.Device', 'DeviceFile')
-        return (utility, recovery)
-    except dbus.DBusException, msg:
-        print "%s, UDisks Failed" % str(msg)
-
-    try:
-        #next try to use devkit-disks. if this fails, then we can fall back to hal
-        obj = bus.get_object('org.freedesktop.DeviceKit.Disks', '/org/freedesktop/DeviceKit/Disks')
-        iface = dbus.Interface(obj, 'org.freedesktop.DeviceKit.Disks')
-        devices = iface.EnumerateDevices()
-        for device in devices:
-            dev_obj = bus.get_object('org.freedesktop.DeviceKit.Disks', device)
-            dev = dbus.Interface(dev_obj, 'org.freedesktop.DBus.Properties')
-
-            label = dev.Get('org.freedesktop.DeviceKit.Disks.Device', 'id-label')
-            filesystem = dev.Get('org.freedesktop.DeviceKit.Disks.Device', 'id-type')
-
-            if not utility and 'DellUtility' in label:
-                utility = dev.Get('org.freedesktop.DeviceKit.Disks.Device', 'device-file')
-            elif not recovery and (('install' in label or 'OS' in label) and 'vfat' in filesystem) or \
-                            ('RECOVERY' in label and 'ntfs' in filesystem):
-                recovery = dev.Get('org.freedesktop.DeviceKit.Disks.Device', 'device-file')
-        return (utility, recovery)
-
-    except dbus.DBusException, msg:
-        print "%s, DeviceKit-Disks Failed" % str(msg)
-
-    try:
-        obj = bus.get_object('org.freedesktop.Hal', '/org/freedesktop/Hal/Manager')
-        iface = dbus.Interface(obj, 'org.freedesktop.Hal.Manager')
-        devices = iface.FindDeviceByCapability('volume')
-
-        for device in devices:
-            dev_obj = bus.get_object('org.freedesktop.Hal', device)
-            dev = dbus.Interface(dev_obj, 'org.freedesktop.Hal.Device')
-
-            label = dev.GetProperty('volume.label')
-            filesystem = dev.GetProperty('volume.fstype')
-            if not utility and 'DellUtility' in label:
-                utility = dev.GetProperty('block.device')
-            elif not recovery and (('install' in label or 'OS' in label) and 'vfat' in filesystem) or \
-                            ('RECOVERY' in label and 'ntfs' in filesystem):
-                recovery = dev.GetProperty('block.device')
-        return (utility, recovery)
-    except dbus.DBusException, msg:
-        print "%s, HAL Failed" % str(msg)
+    utility = find_factory_partition_stats('up')
+    if 'device' in utility:
+        utility = utility['device']
+    else:
+        utility = None
+    recovery = find_factory_partition_stats('rp')
+    if 'device' in recovery:
+        recovery = recovery['device']
+    else:
+        recovery = None
+    return (utility, recovery)
 
 def find_burners():
     """Checks for what utilities are available to burn with"""
@@ -386,49 +331,24 @@ def find_burners():
     #If we have apps for DVD burning, check hardware
     if dvd:
         found_supported_dvdr = False
-        try:
-            bus = dbus.SystemBus()
-            #first try to use udisks, if this fails, fall back to devkit-disks.
-            obj = bus.get_object('org.freedesktop.UDisks', '/org/freedesktop/UDisks')
-            iface = dbus.Interface(obj, 'org.freedesktop.UDisks')
-            devices = iface.EnumerateDevices()
-            for device in devices:
-                obj = bus.get_object('org.freedesktop.UDisks', device)
-                dev = dbus.Interface(obj, 'org.freedesktop.DBus.Properties')
-
-                supported_media = dev.Get('org.freedesktop.UDisks.Device', 'DriveMediaCompatibility')
-                for item in supported_media:
-                    if 'optical_dvd_r' in item:
-                        found_supported_dvdr = True
-                        break
-                if found_supported_dvdr:
+        valid_media_types=[ 'optical_dvd_plus_r', 'optical_dvd_plus_r_dl',
+                            'optical_dvd_plus_rw', 'optical_dvd_r',
+                            'optical_dvd_ram', 'optical_dvd_rw' ]
+        udisks = UDisks.Client.new_sync(None)
+        manager = udisks.get_object_manager()
+        for item in manager.get_objects():
+            drive = item.get_drive()
+            if not drive or not drive.get_cached_property("MediaRemovable"):
+                continue
+            compatibility = drive.get_cached_property("MediaCompatibility")
+            for media in valid_media_types:
+                if media in compatibility:
+                    found_supported_dvdr = True
                     break
-            if not found_supported_dvdr:
-                dvd = None
-            return (dvd, usb)
-        except dbus.DBusException, msg:
-            print "%s, UDisks Failed burner parse" % str(msg)
-        try:
-            #first try to use devkit-disks. if this fails, then, it's OK
-            obj = bus.get_object('org.freedesktop.DeviceKit.Disks', '/org/freedesktop/DeviceKit/Disks')
-            iface = dbus.Interface(obj, 'org.freedesktop.DeviceKit.Disks')
-            devices = iface.EnumerateDevices()
-            for device in devices:
-                obj = bus.get_object('org.freedesktop.DeviceKit.Disks', device)
-                dev = dbus.Interface(obj, 'org.freedesktop.DBus.Properties')
-
-                supported_media = dev.Get('org.freedesktop.DeviceKit.Disks.Device', 'DriveMediaCompatibility')
-                for item in supported_media:
-                    if 'optical_dvd_r' in item:
-                        found_supported_dvdr = True
-                        break
-                if found_supported_dvdr:
-                    break
-            if not found_supported_dvdr:
-                dvd = None
-        except dbus.DBusException, msg:
-            print "%s, device kit Failed burner parse" % str(msg)
-
+            if found_supported_dvdr:
+                break
+        if not found_supported_dvdr:
+            dvd = None
     return (dvd, usb)
 
 def match_system_device(bus, vendor, device):
@@ -499,6 +419,9 @@ def increment_bto_version(version):
 def walk_cleanup(directory):
     """Walks a directory, removes all files, and removes that directory"""
     if os.path.exists(directory):
+        if os.path.isfile(directory):
+            os.remove(directory)
+            return
         for root, dirs, files in os.walk(directory, topdown=False):
             for name in files:
                 os.remove(os.path.join(root, name))
@@ -530,14 +453,14 @@ def create_new_uuid(old_initrd_directory, old_casper_directory,
     #Detect the old initramfs stuff
     try:
         old_initrd_files = glob.glob('%s/initrd*' % old_initrd_directory)
-    except Exception, msg:
-        print str(msg)
-        raise dbus.DBusException, ("Missing initrd in image.")
+    except Exception as msg:
+        logging.warning("create_new_uuid: %s" % str(msg))
+        raise dbus.DBusException("Missing initrd in image.")
     try:
         old_uuid_file   = glob.glob('%s/casper-uuid*' % old_casper_directory)[0]
-    except Exception, msg:
-        print str(msg)
-        raise dbus.DBusException, ("Missing casper UUID in image.")
+    except Exception as msg:
+        logging.warning("create_new_uuid: Old casper UUID not found, assuming 'casper-uuid'")
+        old_uuid_file   = '%s/casper-uuid' % old_casper_directory
 
     old_initrd_file = ''
     old_compression = ''
@@ -557,12 +480,11 @@ def create_new_uuid(old_initrd_directory, old_casper_directory,
             old_suffix = ''
 
     if not old_suffix or not old_initrd_file or not old_uuid_file:
-        raise dbus.DBusException, ("Unable to detect valid initrd.")
+        raise dbus.DBusException("Unable to detect valid initrd.")
 
-    print "Old initrd: %s" % old_initrd_file
-    print "Old uuid file: %s" % old_uuid_file
-    print "Old suffix: %s" % old_suffix
-    print "Old compression method: %s" % old_compression
+    logging.debug("create_new_uuid: old initrd %s, old uuid %s, old suffix %s, \
+old compression method %s" % (old_initrd_file, old_uuid_file,
+                              old_suffix, old_compression))
 
     #Extract old initramfs
     chain0 = subprocess.Popen([old_compression, '-cd', old_initrd_file, '-S',
@@ -573,12 +495,12 @@ def create_new_uuid(old_initrd_directory, old_casper_directory,
     #Generate new UUID
     new_uuid_file = os.path.join(new_casper_directory,
                                  os.path.basename(old_uuid_file))
-    print "New uuid file: %s" % new_uuid_file
+    logging.debug("create_new_uuid: new uuid file: %s" % new_uuid_file)
     chain0 = subprocess.Popen(['uuidgen', '-r'], stdout=subprocess.PIPE)
     new_uuid = chain0.communicate()[0]
-    print "New UUID: %s" % new_uuid.strip()
+    logging.debug("create_new_uuid: new UUID: %s" % new_uuid.strip())
     for item in [new_uuid_file, os.path.join(tmpdir, 'conf', 'uuid.conf')]:
-        with open(item, "w") as uuid_fd:
+        with open(item, "wb") as uuid_fd:
             uuid_fd.write(new_uuid)
 
     #Newer (Ubuntu 11.04+) images may support including the bootstrap in initrd
@@ -595,18 +517,17 @@ def create_new_uuid(old_initrd_directory, old_casper_directory,
     elif new_compression == "auto":
         new_compression = old_compression
         new_suffix = '.' + old_suffix
-    print "New suffix: %s" % new_suffix
-    print "New compression method: %s" % new_compression
+    logging.debug("create_new_uuid: new suffix: %s" % new_suffix)
+    logging.debug("create_new_uuid: new compression method: %s" % new_compression)
 
     #Generate new initramfs
     new_initrd_file = os.path.join(new_initrd_directory, 'initrd' + new_suffix)
-    print "New initrd file: %s" % new_initrd_file
+    logging.debug("create_new_uuid: new initrd file: %s" % new_initrd_file)
     chain0 = subprocess.Popen(['find'], cwd=tmpdir, stdout=subprocess.PIPE)
-    chain1 = subprocess.Popen(['cpio', '--quiet', '--dereference', '-o',
-                               '-H', 'newc'],
+    chain1 = subprocess.Popen(['cpio', '--quiet', '-o', '-H', 'newc'],
                                cwd=tmpdir, stdin=chain0.stdout,
                                stdout=subprocess.PIPE)
-    with open(new_initrd_file, 'w') as initrd_fd:
+    with open(new_initrd_file, 'wb') as initrd_fd:
         if new_compression:
             chain2 = subprocess.Popen([new_compression, '-9c'],
                                       stdin=chain1.stdout,
@@ -623,24 +544,20 @@ def create_g2ldr(chroot, rp_mount, install_mount):
        chroot: chroot to launch commands in
        rp_mount: mountpoint to find partition# and install g2ldr
        install_mount: mountpoint to find target OS UUID'''
-    bus = dbus.SystemBus()
-    udisk_bus_name = 'org.freedesktop.UDisks'
-    dev_bus_name   = 'org.freedesktop.UDisks.Device'
     uuid = ''
     partition = '-1'
 
-    #Find the UUID we are installing to
-    obj = bus.get_object(udisk_bus_name, '/org/freedesktop/UDisks')
-    iface = dbus.Interface(obj, udisk_bus_name)
-    devices = iface.EnumerateDevices()
-    for device in devices:
-        obj = bus.get_object(udisk_bus_name, device)
-        dev = dbus.Interface(obj, 'org.freedesktop.DBus.Properties')
-        mount = dev.Get(dev_bus_name, 'DeviceMountPaths')
+    udisks = UDisks.Client.new_sync(None)
+    manager = udisks.get_object_manager()
+    for item in manager.get_objects():
+        filesystem = item.get_filesystem()
+        part = item.get_partition()
+        if not filesystem or not part:
+            continue
+        mount = filesystem.get_cached_property('MountPoints')
         if mount and install_mount and install_mount in mount:
-            uuid = dev.Get(dev_bus_name, 'IdUuid')
-        elif mount and rp_mount and rp_mount in mount:
-            partition = str(dev.Get(dev_bus_name, 'DeviceMinor'))
+            uuid = part.get_cached_property('UUID')
+            partition = part.get_cached_property("Number")
 
     #The file that the Windows BCD will chainload
     shutil.copy('/usr/lib/grub/i386-pc/g2ldr.mbr', rp_mount)
@@ -669,9 +586,9 @@ def create_g2ldr(chroot, rp_mount, install_mount):
         from ubiquity import misc
         misc.execute_root(*build_command)
 
-    with open(os.path.join(rp_mount, 'g2ldr'), 'w') as wfd:
+    with open(os.path.join(rp_mount, 'g2ldr'), 'wb') as wfd:
         for fname in ['/usr/lib/grub/i386-pc/g2hdr.bin', os.path.join(chroot, 'tmp', 'core.img')]:
-            with open(fname) as rfd:
+            with open(fname, 'rb') as rfd:
                 wfd.write(rfd.read())
 
 def parse_seed(seed):
@@ -720,6 +637,61 @@ def write_up_bootsector(device, partition):
     else:
         fetch_output(['dosfslabel', device + partition, "DellUtility"])
 
+def create_up_boot_entry(new_text):
+    upart = find_factory_partition_stats('up')
+
+    if not os.path.isdir('/sys/firmware/efi') and upart:
+        process_conf_file(original = '/usr/share/dell/grub/98_dell_bios', \
+                                new = '/etc/grub.d/98_dell_bios',               \
+                                uuid = str(upart["uuid"]),                      \
+                                number = str(upart["number"]),                  \
+                                recovery_text = new_text)
+
+        os.chmod('/etc/grub.d/98_dell_bios', 0o755)
+
+def mark_upgrades():
+    '''Mark packages that can upgrade to upgrade during install'''
+    from apt.cache import Cache
+    cache = Cache()
+    to_install = []
+    for key in cache.keys():
+        if cache[key].is_upgradable:
+            to_install.append(key)
+    del cache
+    return to_install
+
+def mark_unconditional_debs(add_directory=''):
+    '''Finds any debs from debs/main that we want unconditionally installed
+       (but ONLY the latest version on the media)'''
+    import apt_inst
+    import apt_pkg
+
+    def parse(fname):
+        """ read a deb """
+        control = apt_inst.DebFile(fname).control.extractdata("control")
+        sections = apt_pkg.TagSection(control)
+        if "Modaliases" in sections:
+            modaliases = sections["Modaliases"]
+        else:
+            modaliases = ''
+        return (sections["Architecture"], sections["Package"], modaliases)
+
+    #process debs/main
+    to_install = []
+    my_arch = fetch_output(['dpkg', '--print-architecture']).strip()
+    for top in [ISO_MOUNT, CDROM_MOUNT, add_directory]:
+        repo = os.path.join(top, 'debs', 'main')
+        if os.path.isdir(repo):
+            for fname in os.listdir(repo):
+                if '.deb' in fname:
+                    arch, package, modaliases = parse(os.path.join(repo, fname))
+                    if not modaliases and (arch == "all" or arch == my_arch):
+                        to_install.append(package)
+
+    #These aren't in all images, but desirable if available
+    to_install.append('dkms')
+
+    return to_install
 
 def dbus_sync_call_signal_wrapper(dbus_iface, func, handler_map, *args, **kwargs):
     '''Run a D-BUS method call while receiving signals.
@@ -746,14 +718,14 @@ def dbus_sync_call_signal_wrapper(dbus_iface, func, handler_map, *args, **kwargs
         _h_exception_exc = exception
         loop.quit()
 
-    loop = gobject.MainLoop()
+    loop = GLib.MainLoop()
     global _h_reply_result, _h_exception_exc
     _h_reply_result = None
     _h_exception_exc = None
     kwargs['reply_handler'] = _h_reply
     kwargs['error_handler'] = _h_error
     kwargs['timeout'] = 86400
-    for signame, sighandler in handler_map.iteritems():
+    for signame, sighandler in handler_map.items():
         dbus_iface.connect_to_signal(signame, sighandler)
     dbus_iface.get_dbus_method(func)(*args, **kwargs)
     loop.run()
